module ActsAsMailable
  # including this module into your EmailTemplate model will give you finders and named scopes
  # useful for working with EmailTemplates.
  # The named scopes are:
  #   in_order: Returns email_templates in the order they were created (created_at ASC).
  #   recent: Returns email_templates by how recently they were created (created_at DESC).
  #   limit(N): Return no more than N email_templates.
  module EmailTemplate

    def self.included(email_template_model)
      email_template_model.extend Finders
      email_template_model.scope :in_order, -> { email_template_model.order('created_at ASC') }
      email_template_model.scope :recent, -> { email_template_model.reorder('created_at DESC') }
    end

    module Finders
      # Helper class method to look up all email_templates for
      # mailable class name and mailable id.
      def find_email_templates_for_mailable(mailable_str, mailable_id)
        where(["mailable_type = ? and mailable_id = ?", mailable_str, mailable_id]).order("created_at DESC")
      end

      # Helper class method to look up a mailable object
      # given the mailable class name and id
      def find_mailable(mailable_str, mailable_id)
        model = mailable_str.constantize
        model.respond_to?(:find_email_templates_for) ? model.find(mailable_id) : nil
      end
    end
  end
end

